<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:w2e="http://thisSheet" xmlns:xsd="http://www.w3.org/2001/XMLSchema">

<!-- CAM utility functions -->
<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
     Authors: Martin ME Roberts and David RR Webber, March, 2008 -->
<!--
    Purpose  : Parse XSD schema into CAM template XML structure components.
               Creates XML instance to be used as Structure in CAM template editor.
    Original : 03/21/2008
    Initials : mmer
    CAM ver    : 1.1 
    Description : Part of a set of tools to ease alignment of XSD based validations and documentation
                  and CAM templates.
    Updated  - Inits - Change
    03/21/08   mmer    Initial release
    03/22/08   drrw    Added extended type handling
    03/22/08   drrw    Added type reference handling with namespace 
    03/22/08   drrw    Added lookup of enumeration value for type example
    03/22/08   drrw    Added entity type referencing
    03/24/08   drrw    Added type handling for duration and gYear/Month/Day
    03/24/08   drrw    Add attribute type handling
    03/24/08   mmer  Added option for CAM output or not
    03/25/08   drrw    Tweaked namespace handling for default namespaces
    03/25/08   drrw    Added new parameter for instance, rules, inline, reference and anno generation
    03/27/08   drrw    Added xsd:normalizedString and dateTime types
    03/27/08   drrw    Added ID and IDREF types
    03/31/08   drrw    Reverted to structure instance output only
    04/01/08   drrw    Added positive integer
    04/01/08   mmer    Structure mode template matching
    04/02/08   drrw      Add text() to structure to pick up %values%
    04/11/08   drrw    Suppress quotes in enumerated values if not CAM
    04/11/08   drrw    When not CAM mode - put in actual data value for date / times
    04/14/08   drrw    Remove Z quotes in date/time masks
    04/14/08   drrw    date and dateTime example generation refined
    04/16/08   drrw    Duration example data value P1 from P1Y2M3DT1H10M0S
    04/16/08   drrw    xsd:int data value
    04/17/08   drrw    Comment out non-essential debugging messages
    04/21/08   drrw    xs:language and xs:token type support
    05/08/08   drrw    add default namespace uri to asm1 definition if present
    08/02/08   drrw    add float type and support for int and integer unprefixed
    06/23/08   mmer cast $CAM to boolean
    02/04/09   drrw    type-to-example decimal or float with no fraction digits ignored
    03/30/09   drrw    namespace cleanup fix
    05/15/09   drrw    change float example value
    03/01/10   drrw    change log message prefix from error to message
    12/02/10   drrw    type-to-example add QName
    01/05/12   drrw    minor tweak to default anyURL value
-->

	<xsl:template name="namespace-uri-of-qname">
		<xsl:param name="qname"/>
		<xsl:param name="context"/>
		<!-- xsl:value-of select="w2e:message(concat('namespace-uri-of-qname(',$qname,')'),4)"/ -->
		<xsl:variable name="results">
		<xsl:choose>
			<xsl:when test="string-length($context) &gt; 0">
				<xsl:if test="contains($qname,':')">
					<xsl:value-of select="distinct-values($context//namespace::*[name()=substring-before($qname,':')])"/>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="contains($qname,':')">
					<xsl:value-of select="distinct-values(namespace::*[name()=substring-before($qname,':')])"/>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
		</xsl:variable>
		<xsl:variable name="namespace" select="if (string-length($results) = 0) then ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace else $results"/>
		<!-- xsl:value-of select="w2e:message(concat('namespace = ',$namespace),4)"/ -->
		<xsl:value-of select="$namespace"/>
	</xsl:template>

	<xsl:template name="prefix-for-namespace">
		<xsl:param name="namespace-uri"/>
		<xsl:param name="context"/>
		<!-- xsl:value-of select="w2e:message(concat('prefix-for-namespace(',$namespace-uri,')'),4)"/ -->
		<xsl:if test="$namespace-uri">
			<xsl:variable name="prefix">
			<xsl:choose>
				<xsl:when test="string-length($context) &gt; 0">
					<!-- terminate if current() does not have a prefix for $namespace-uri -->
					<xsl:if test="not($context//namespace::*[string() = $namespace-uri])">
						<xsl:value-of select="w2e:message(
						concat('Unable to find namespace prefix for namespace ',$namespace-uri,' while processing ',name(),' element.'),0)"/>
					</xsl:if>
					<xsl:value-of select="distinct-values($context//namespace::*[string() = $namespace-uri]/name())"/>
				</xsl:when>
				<xsl:otherwise>
					<!-- terminate if current() does not have a prefix for $namespace-uri -->
					<xsl:variable name="namespaces">
						<xsl:sequence select="namespace::*[string() = $namespace-uri]/name()"/>
					</xsl:variable>
					<xsl:if test="not($namespaces[position()=1])">
							<xsl:value-of select="w2e:message(
						concat('Unable to find namespace prefix for namespace ',$namespace-uri,' while processing ',name(),' element.'),1)"/>
					</xsl:if>
					<xsl:value-of select="replace($namespaces[position()=1],' ','')" />
				</xsl:otherwise>
			</xsl:choose>
			<xsl:text>:</xsl:text>
			</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('prefix=',$prefix),4)"/ -->
		<xsl:value-of select="$prefix"/>
		</xsl:if>

	</xsl:template>

	<xsl:function name="w2e:type-to-example">
		<xsl:param name="type" as="xsd:string"/>
		<xsl:param name="refType" as="xsd:string"/>
		<xsl:variable name="timeSep">'T'</xsl:variable>
		<xsl:variable name="typeName" select="concat($type,'|')"></xsl:variable>
		<xsl:variable name="percent" select="if (xsd:boolean($CAM)) then '%' else ''"/>
		<xsl:choose>
		<xsl:when test="string-length($type) = 0"><xsl:value-of select="$percent"/>xs:anyType<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($type,'&quot;')"><xsl:value-of select="$percent"/><xsl:value-of select="if(xsd:boolean($CAM)) then $type else substring-before(substring-after($type,'&quot;'),'&quot;')"/><xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'string|')"><xsl:value-of select="$percent"/>string<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'normalizedString|')"><xsl:value-of select="$percent"/>string<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="$typeName = 'ID|'"><xsl:value-of select="$percent"/>ID-12345<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="$typeName = 'IDREFS|'"><xsl:value-of select="$percent"/>IDREFS-12345<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="$typeName = 'IDREF|'"><xsl:value-of select="$percent"/>IDREF-12345<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'dateTime|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>YYYY-MM-DD'T'HH:MI:SS.SZ<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]T[h01]:[m01]:[s01].0[Zn]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'date|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>YYYY-MM-DDZ<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01][Zn]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'gYear|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>YYYY<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'gYearMonth|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>YYYY-MM<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'gMonth|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>--MM<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'--[M01]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'gMonthDay|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>--MM-DD<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'--[M01]-[D01]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'gDay|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>---DD<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'---[D01]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'time|')"><xsl:choose><xsl:when test="xsd:boolean($CAM)"><xsl:value-of select="$percent"/>HH:MI:SS.SZ<xsl:value-of select="$percent"/></xsl:when><xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].0[Zn]')"/></xsl:otherwise></xsl:choose></xsl:when>
		<xsl:when test="contains($typeName,'string|')"><xsl:value-of select="$percent"/>string<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'integer|') or(contains($typeName,':int|'))"><xsl:value-of select="$percent"/>12345<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="$type='int' or($type='integer')"><xsl:value-of select="$percent"/>12345<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'decimal|')"><xsl:value-of select="$percent"/>54321.00<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'float|')"><xsl:value-of select="$percent"/>4321.000<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'anyURI|')"><xsl:value-of select="$percent"/><xsl:value-of select="if (xsd:boolean($CAM)) then 'http://wiki.oasis-open.org/cam' else 'http://anywhereURL.org/home' "/><xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'nonNegativeInteger|')"><xsl:value-of select="$percent"/>1<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'positiveInteger|')"><xsl:value-of select="$percent"/>1<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'boolean|')"><xsl:value-of select="$percent"/>false<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'duration|')"><xsl:value-of select="$percent"/>P1<xsl:value-of select="$percent"/></xsl:when>	
		<xsl:when test="contains($typeName,'language|')"><xsl:value-of select="$percent"/>en-US<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'token|')"><xsl:value-of select="$percent"/>Token<xsl:value-of select="$percent"/></xsl:when>
		<xsl:when test="contains($typeName,'QName|')"><xsl:value-of select="$percent"/>QName<xsl:value-of select="$percent"/></xsl:when>
		<!-- xsl:when test="string-length($entityValue) > 2"><xsl:value-of select="$percent"/><xsl:value-of select="$entityValue"/><xsl:value-of select="$percent"/></xsl:when -->
		<xsl:when test="$refType = 'attr'"><xsl:value-of select="$percent"/><xsl:value-of select="$type"/><xsl:value-of select="$percent"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="$percent"/><xsl:value-of select="$refType"/>=<xsl:value-of select="$type"/><xsl:value-of select="$percent"/></xsl:otherwise>
		</xsl:choose>
	</xsl:function>

	<xsl:function name="w2e:message">
		<xsl:param name="text" as="xsd:string"/>
		<xsl:param name="level" as="xsd:integer"/>
		<xsl:variable name="loglevel" as="xsd:integer" select="xsd:integer($logLevel)"/>
		<xsl:choose>
		<xsl:when test="$level=0 and $loglevel &gt;=0">
			<xsl:message terminate="yes">[fatal]<xsl:value-of select="$text"/></xsl:message>
		</xsl:when>
		<xsl:when test="$level=1 and $loglevel &gt;=1">
			<xsl:message>[message]<xsl:value-of select="$text"/></xsl:message>
		</xsl:when>
		<xsl:when test="$level=2 and $loglevel &gt;=2">
			<xsl:message>[warn]<xsl:value-of select="$text"/></xsl:message>
		</xsl:when>
		<xsl:when test="$level=3 and $loglevel &gt;=3">
			<xsl:message>[info]<xsl:value-of select="$text"/></xsl:message>
		</xsl:when>
		<xsl:when test="$level=4 and $loglevel &gt;=4">
			<xsl:message>[debug]<xsl:value-of select="$text"/></xsl:message>
		</xsl:when>
		</xsl:choose>
	</xsl:function>

	<xsl:template name="getNs">
		<xsl:param name="context"/>
	<xsl:variable name="ns">
		<xsl:sequence select="distinct-values($context//*/namespace-uri())"/>
	</xsl:variable>
	<xsl:value-of select="w2e:message(concat('namespaces: ',$ns),4)"/>
	<nss>
		<xsl:for-each select="tokenize($ns,' ')">
		<ns prefix="{concat('a',position())}" uri="{.}"/>
		</xsl:for-each>
		<xsl:choose>
			<xsl:when test="string-length($defaultNS)=0"><ns prefix="asm1" uri="http://www.oasis-open.org/committees/cam/content"/></xsl:when>
			<xsl:otherwise><ns prefix="asm1" uri="{$defaultNS}"/></xsl:otherwise>
		</xsl:choose>
	</nss>
	</xsl:template>

	<xsl:template name="correctNamespaces">
		<xsl:param name="nss"/>
		<xsl:apply-templates select="$nss" mode="ns"/>
	</xsl:template>
		
	<xsl:template match="*" mode="ns">
		<xsl:param name="nss"/>
		<xsl:variable name="uri" select="namespace-uri()"/>
		<!-- xsl:message>uri:<xsl:value-of select="$uri"/></xsl:message -->
		<xsl:variable name="prefix">
			<xsl:choose>
				<xsl:when test="contains(name(),':')"><xsl:value-of select="substring-before(name(),':')"/></xsl:when>
				<xsl:otherwise><!-- xsl:value-of select="$nss/nss/ns[@uri=$uri]/@prefix"/--></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:message>[<xsl:value-of select="name()"/>:<xsl:value-of select="$prefix"/>]</xsl:message -->
		
		<xsl:variable name="name">
			<xsl:choose>
				<xsl:when test=" string-length($prefix) &gt; 0"><xsl:value-of select="concat($prefix,':',local-name())"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="local-name()"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
				<xsl:choose>
				<xsl:when test="contains($name,':')"><!-- or(string-length($uri) &gt; 0) -->
					<xsl:element name="{$name}" namespace="{$uri}">
					<xsl:apply-templates select="@*" mode="ns"/>
					<xsl:apply-templates mode="ns">
						<xsl:with-param name="nss" select="$nss"/>
					</xsl:apply-templates>
					</xsl:element>
				</xsl:when>
				<xsl:when test="count($nss//ns[@prefix='asm1']) &gt; 0">
					<xsl:variable name="asmNS" select="$nss//ns[@prefix='asm1']/@uri"></xsl:variable>
					<!-- xsl:message>[asm1: <xsl:value-of select="$asmNS"/>]</xsl:message -->
					<xsl:element name="{$name}"><!-- "asm1:{$name}" namespace="{$asmNS}" -->
						<xsl:apply-templates select="@*" mode="ns">
							<xsl:with-param name="asmNS" select="'asm1:'"/>
						</xsl:apply-templates>
						<xsl:apply-templates mode="ns">
							<xsl:with-param name="nss" select="$nss"/>
						</xsl:apply-templates>
					</xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:element name="{$name}">
						<xsl:apply-templates select="@*" mode="ns"/>
						<xsl:apply-templates mode="ns">
							<xsl:with-param name="nss" select="$nss"/>
						</xsl:apply-templates>
					</xsl:element>
				</xsl:otherwise>
				</xsl:choose>
	</xsl:template>
	
	<xsl:template match="@*" mode="ns">
		<xsl:param name="asmNS"/>
		<xsl:copy-of select="."/>
		<!-- xsl:choose>
			<xsl:when test="$asmNS=''"><xsl:copy-of select="."/></xsl:when>
			<xsl:when test="contains(name(.),':')"><xsl:copy-of select="."/></xsl:when>
			<xsl:otherwise><xsl:copy-of select="." xpath-default-namespace="{$asmNS}"></xsl:copy-of></xsl:otherwise>
		</xsl:choose -->
	</xsl:template>
	<xsl:template match="text()" mode="ns">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="*" mode="structure">
		<xsl:copy>
			<xsl:apply-templates select="@*" mode="structure"/>
			<xsl:apply-templates select="*" mode="structure"/>
			<xsl:apply-templates select="text()" mode="structure"/>
			<xsl:apply-templates select="comment()" mode="structure"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="xsd:*" mode="structure">
		<xsl:apply-templates select="@*|*" mode="structure"/>
	</xsl:template>

	<xsl:template match="@xsd:*" mode="structure"/>

	<xsl:template match="@*|text()|comment()" mode="structure">
		<xsl:copy-of select="."/>
	</xsl:template>

</xsl:stylesheet>
